import { Host, h, } from "@stencil/core";
import { IcBrandForegroundEnum, } from "../../utils/types";
import { slotHasContent, getBrandForegroundAppearance, onComponentRequiredPropUndefined, isPropDefined, isSlotUsed, renderDynamicChildSlots, } from "../../utils/helpers";
/**
 * @slot heading - Content will be rendered in the title area, in place of the heading.
 * @slot subheading - Content will be rendered in the title area, in place of the subheading.
 * @slot interaction - Content will be rendered in the interaction area, adjacent to the title area.
 * @slot secondary - Content will be rendered in the secondary content area.
 */
export class Hero {
    constructor() {
        this.hostMutationObserver = null;
        this.foregroundColor = getBrandForegroundAppearance();
        this.rightContent = false;
        // set by above state
        this.leftContentFullWidth = !this.rightContent && this.secondaryHeading === undefined;
        this.scrollFactor = "right -100px";
        /**
         * The alignment of the hero.
         */
        this.aligned = "left";
        /**
         * The alignment of the hero content.
         */
        this.contentAligned = "left";
        /**
         * If `true`, the background image (if set) will not scroll using a parallax effect.
         */
        this.disableBackgroundParallax = false;
        /**
         * The size of the hero component.
         */
        this.size = "medium";
    }
    disconnectedCallback() {
        var _a;
        (_a = this.hostMutationObserver) === null || _a === void 0 ? void 0 : _a.disconnect();
    }
    componentWillLoad() {
        this.rightContent = slotHasContent(this.el, "secondary");
    }
    componentDidLoad() {
        !isSlotUsed(this.el, "heading") &&
            onComponentRequiredPropUndefined([{ prop: this.heading, propName: "heading" }], "Hero");
        this.hostMutationObserver = new MutationObserver((mutationList) => renderDynamicChildSlots(mutationList, "secondary", this));
        this.hostMutationObserver.observe(this.el, {
            childList: true,
        });
    }
    componentWillRender() {
        this.rightContent = slotHasContent(this.el, "secondary");
    }
    brandChangeHandler(ev) {
        this.foregroundColor = ev.detail.mode;
    }
    doScroll() {
        var _a;
        if (this.disableBackgroundParallax) {
            return;
        }
        const scrolltotop = (_a = document.scrollingElement) === null || _a === void 0 ? void 0 : _a.scrollTop;
        if (scrolltotop) {
            const factor = 0.4;
            const y = -100 + scrolltotop * factor;
            this.scrollFactor = "right " + y + "px";
        }
    }
    render() {
        const { aligned, size, heading, subheading, secondaryHeading, secondarySubheading, foregroundColor, backgroundImage, scrollFactor, } = this;
        let style = {};
        if (isPropDefined(backgroundImage)) {
            style = {
                "background-image": "url(" + backgroundImage + ")",
                "background-position": scrollFactor,
            };
        }
        return (h(Host, { key: '4c88b4471d3485baff1f6c55f859639feb3166d8', class: {
                [`ic-hero-${IcBrandForegroundEnum.Dark}`]: foregroundColor === IcBrandForegroundEnum.Dark,
                ["has-background-image"]: backgroundImage !== undefined,
                ["has-right-content"]: this.rightContent,
                ["ic-hero-small"]: size === "small",
                ["secondary-heading"]: !!secondaryHeading,
            }, style: style }, h("ic-section-container", { key: 'b68716e353d9b2767dc16a5f701e3cdcbd32c748', aligned: aligned, fullHeight: true, class: "section-container" }, h("div", { key: '6dd6f422c0bd36725be4b41b42633aeebfa1911d', class: {
                ["left-container"]: true,
                ["left-container-full-width"]: this.leftContentFullWidth,
            } }, h("div", { key: '54ffef99b79eef5eedbf1571bff89270982b5671', class: "heading" }, h("slot", { key: '6b7f71caecbaa84d1a39295e100aedef32d1d522', name: "heading" }, h("ic-typography", { key: '841b429a53b8b72dd60017cb2b2404b826f5bbe8', variant: size === "small" ? "h2" : "h1", class: {
                ["heading-bottom-spacing"]: size !== "small",
            } }, h("h2", { key: 'ee802d396bb8a5638f2ab8a62fddbf764436e20e' }, heading)))), h("div", { key: '9a051783a24c9a92fc0aeff2db50689befe4834e', class: "subheading" }, h("slot", { key: 'eb894eaeb4b4303be0caf090d85d487aa11284fb', name: "subheading" }, h("ic-typography", { key: '92b4d9af6118e2ff9bf2b9166862067c610fffee', variant: "body" }, subheading))), h("div", { key: 'cacf81a4666eba19211493914e9b1af466e61478', class: "interaction-container" }, h("slot", { key: '982fb4b528bea4c486d9c0df2cf47fb3ace19414', name: "interaction" }))), (isPropDefined(secondaryHeading) || this.rightContent) && (h("div", { key: '32cd6196da302dd5d0896c90d53d1b35c060e31c', class: "right-container" }, h("slot", { key: '655f0a9978d8402faab43e7e143987f2934255f6', name: "secondary" }, secondaryHeading && (h("div", { key: '94de7e79fd9122b4084b57bf0b5b0571d77e7adc', class: "secondary-container" }, h("div", { key: 'afd414fd37f9b1c8fb6d9f2d7957314f35a1da40', class: "secondary-heading" }, h("ic-typography", { key: 'd643e471ec716d3e6f33ec7761027323ca31bc8c', variant: "h4" }, h("h3", { key: 'f6296bbd33d2ca70beb5100151cebfa8732206d0' }, secondaryHeading))), h("div", { key: '0575ef33984c79674626d9f114ad90a3f973e63a', class: "secondary-subheading" }, h("ic-typography", { key: '8045e445231e4cbbd07ad9fa21a7b6bbf5015eff', variant: "subtitle-small" }, secondarySubheading))))))))));
    }
    static get is() { return "ic-hero"; }
    static get encapsulation() { return "shadow"; }
    static get originalStyleUrls() {
        return {
            "$": ["ic-hero.css"]
        };
    }
    static get styleUrls() {
        return {
            "$": ["ic-hero.css"]
        };
    }
    static get properties() {
        return {
            "aligned": {
                "type": "string",
                "mutable": false,
                "complexType": {
                    "original": "IcAlignment",
                    "resolved": "\"center\" | \"full-width\" | \"left\" | undefined",
                    "references": {
                        "IcAlignment": {
                            "location": "import",
                            "path": "../../utils/types",
                            "id": "src/utils/types.ts::IcAlignment"
                        }
                    }
                },
                "required": false,
                "optional": true,
                "docs": {
                    "tags": [],
                    "text": "The alignment of the hero."
                },
                "getter": false,
                "setter": false,
                "attribute": "aligned",
                "reflect": false,
                "defaultValue": "\"left\""
            },
            "backgroundImage": {
                "type": "string",
                "mutable": false,
                "complexType": {
                    "original": "string",
                    "resolved": "string | undefined",
                    "references": {}
                },
                "required": false,
                "optional": true,
                "docs": {
                    "tags": [],
                    "text": "The optional background image."
                },
                "getter": false,
                "setter": false,
                "attribute": "background-image",
                "reflect": false
            },
            "contentAligned": {
                "type": "string",
                "mutable": false,
                "complexType": {
                    "original": "IcHeroContentAlignments",
                    "resolved": "\"center\" | \"left\" | undefined",
                    "references": {
                        "IcHeroContentAlignments": {
                            "location": "import",
                            "path": "./ic-hero.types",
                            "id": "src/components/ic-hero/ic-hero.types.tsx::IcHeroContentAlignments"
                        }
                    }
                },
                "required": false,
                "optional": true,
                "docs": {
                    "tags": [],
                    "text": "The alignment of the hero content."
                },
                "getter": false,
                "setter": false,
                "attribute": "content-aligned",
                "reflect": false,
                "defaultValue": "\"left\""
            },
            "disableBackgroundParallax": {
                "type": "boolean",
                "mutable": false,
                "complexType": {
                    "original": "boolean",
                    "resolved": "boolean | undefined",
                    "references": {}
                },
                "required": false,
                "optional": true,
                "docs": {
                    "tags": [],
                    "text": "If `true`, the background image (if set) will not scroll using a parallax effect."
                },
                "getter": false,
                "setter": false,
                "attribute": "disable-background-parallax",
                "reflect": false,
                "defaultValue": "false"
            },
            "heading": {
                "type": "string",
                "mutable": false,
                "complexType": {
                    "original": "string",
                    "resolved": "string | undefined",
                    "references": {}
                },
                "required": false,
                "optional": true,
                "docs": {
                    "tags": [],
                    "text": "The heading of the hero. An <h2> level heading."
                },
                "getter": false,
                "setter": false,
                "attribute": "heading",
                "reflect": false
            },
            "secondaryHeading": {
                "type": "string",
                "mutable": false,
                "complexType": {
                    "original": "string",
                    "resolved": "string | undefined",
                    "references": {}
                },
                "required": false,
                "optional": true,
                "docs": {
                    "tags": [],
                    "text": "The optional secondary heading, an <h3> level heading. Replaced by slotted right content."
                },
                "getter": false,
                "setter": false,
                "attribute": "secondary-heading",
                "reflect": false
            },
            "secondarySubheading": {
                "type": "string",
                "mutable": false,
                "complexType": {
                    "original": "string",
                    "resolved": "string | undefined",
                    "references": {}
                },
                "required": false,
                "optional": true,
                "docs": {
                    "tags": [],
                    "text": "The optional secondary subheading, replaced by slotted right content."
                },
                "getter": false,
                "setter": false,
                "attribute": "secondary-subheading",
                "reflect": false
            },
            "size": {
                "type": "string",
                "mutable": false,
                "complexType": {
                    "original": "IcSizesNoLarge",
                    "resolved": "\"medium\" | \"small\" | undefined",
                    "references": {
                        "IcSizesNoLarge": {
                            "location": "import",
                            "path": "../../utils/types",
                            "id": "src/utils/types.ts::IcSizesNoLarge"
                        }
                    }
                },
                "required": false,
                "optional": true,
                "docs": {
                    "tags": [],
                    "text": "The size of the hero component."
                },
                "getter": false,
                "setter": false,
                "attribute": "size",
                "reflect": false,
                "defaultValue": "\"medium\""
            },
            "subheading": {
                "type": "string",
                "mutable": false,
                "complexType": {
                    "original": "string",
                    "resolved": "string | undefined",
                    "references": {}
                },
                "required": false,
                "optional": true,
                "docs": {
                    "tags": [],
                    "text": "The description for the hero."
                },
                "getter": false,
                "setter": false,
                "attribute": "subheading",
                "reflect": false
            }
        };
    }
    static get states() {
        return {
            "foregroundColor": {},
            "rightContent": {},
            "leftContentFullWidth": {},
            "scrollFactor": {}
        };
    }
    static get elementRef() { return "el"; }
    static get listeners() {
        return [{
                "name": "brandChange",
                "method": "brandChangeHandler",
                "target": "document",
                "capture": false,
                "passive": false
            }, {
                "name": "scroll",
                "method": "doScroll",
                "target": "document",
                "capture": false,
                "passive": true
            }];
    }
}
//# sourceMappingURL=ic-hero.js.map
