import { h, Host } from "@stencil/core";
const LOADING_INDICATOR_WIDTHS = {
    large: 120,
    medium: 80,
    small: 40,
    icon: 20,
};
export class LoadingIndicator {
    constructor() {
        this.labelList = [];
        this.circularDiameter = 0;
        this.circularLineWidth = 0;
        this.circularDimensions = {
            x: 0,
            y: 0,
            r: 0,
        };
        this.clipInnerElement = false;
        /**
         * The description that will be set as the aria-label of the loading indicator when not using a visible label.
         */
        this.description = "Loading";
        /**
         * If `true`, when linear, the full-width variant (i.e. without a border radius) will be displayed.
         */
        this.fullWidth = false;
        /**
         * The time in milliseconds before the label changes.
         */
        this.labelDuration = 8000;
        /**
         * The maximum value that the progress value can take.
         * Used to calculate the proportional width of the progress bar.
         */
        this.max = 100;
        /**
         * The minimum value that the progress value can take.
         * Used to calculate the proportional width of the progress bar.
         */
        this.min = 0;
        /**
         * If `true`, the element will display as black and white.
         */
        this.monochrome = false;
        /**
         * The size of the loading indicator.
         */
        this.size = "medium";
        /**
         * Sets the theme color to the dark or light theme color. "inherit" will set the color based on the system settings or ic-theme component.
         */
        this.theme = "inherit";
        /**
         * The type of indicator, either linear or circular.
         */
        this.type = "circular";
        this.getLabel = (labelIndex) => new Promise(() => {
            this.interval = setInterval(() => {
                if (labelIndex < this.labelList.length - 1) {
                    labelIndex++;
                }
                else {
                    labelIndex = 0;
                }
                this.indicatorLabel = this.labelList[labelIndex];
            }, this.labelDuration);
        });
        this.getLabelVariant = () => {
            const width = this.type === "circular" ? this.calculateWidth() : 0;
            if (this.size === "small" || (width && width < 60)) {
                return "label";
            }
            else if (this.size === "large" || width >= 120) {
                return "h2";
            }
            return "h4";
        };
        this.calculateWidth = () => {
            var _a, _b, _c;
            if ((_a = this.outerElement) === null || _a === void 0 ? void 0 : _a.offsetWidth)
                return this.outerElement.offsetWidth;
            if (this.outerElement) {
                const { width } = window.getComputedStyle(this.outerElement);
                if (width) {
                    // Regex to check the units of the css string
                    if (((_b = width.match(/\D+$/)) === null || _b === void 0 ? void 0 : _b[0]) !== "%")
                        return parseFloat(width);
                    const parentWidth = (_c = this.outerElement.parentElement) === null || _c === void 0 ? void 0 : _c.offsetWidth;
                    if (parentWidth)
                        return parentWidth * (parseFloat(width) / 100);
                }
            }
            return LOADING_INDICATOR_WIDTHS[this.size];
        };
        this.setLinearDeterminateWidth = () => {
            // Ensure progress cannot be out of bounds
            if (!this.innerElement || !this.progress)
                return;
            const progress = Math.min(this.max, Math.max(this.min, this.progress));
            const proportion = (progress - this.min) / (this.max - this.min);
            this.clipInnerElement = proportion > 0.5;
            if (this.clipInnerElement) {
                this.innerElement.classList.remove("clip");
            }
            else {
                this.innerElement.classList.add("clip");
            }
            this.innerElement.style.setProperty("--linear-width", `${proportion * 100}%`);
        };
        this.updateLabel = () => {
            if (!this.label)
                return;
            if (typeof this.label === "string") {
                this.indicatorLabel = this.label;
            }
            else {
                this.labelList = this.label;
                this.indicatorLabel = this.labelList[0];
                if (this.labelList.length > 1) {
                    this.getLabel(0);
                }
            }
        };
        this.setCircleDimensions = () => {
            let dashOffset = undefined;
            if (this.circularDiameter <= 0)
                return;
            const r = this.circularDiameter / 2;
            const nextRadius = r - this.circularLineWidth / 2;
            const dashArray = 2 * Math.PI * nextRadius;
            if (this.progress) {
                const progress = Math.min(Math.max(this.progress, this.min), this.max);
                dashOffset = `${(-1 - (progress - this.min) / (this.max - this.min)) * dashArray}px`;
            }
            this.circularDimensions = {
                x: r,
                y: r,
                r: nextRadius,
                dashArray: `${dashArray}px`,
                dashOffset,
            };
        };
    }
    watchPropHandler() {
        this.updateLabel();
    }
    minMaxChangeHandler() {
        if (this.type === "circular")
            this.setCircleDimensions();
    }
    watchProgressHandler() {
        if (this.type === "linear") {
            this.setLinearDeterminateWidth();
        }
        else {
            this.setCircleDimensions();
        }
    }
    setIndicatorDimensions() {
        if (this.type === "circular") {
            // Sets the circular indicator line width - accounting for the circle size being altered using the CSS custom property
            const diameter = this.calculateWidth();
            if (this.outerElement && diameter !== this.circularDiameter) {
                this.circularLineWidth = diameter * 0.1;
                this.circularDiameter = diameter;
                this.outerElement.style.setProperty("--circular-line-width", `${this.circularLineWidth}px`);
                this.setCircleDimensions();
            }
        }
        else {
            this.setLinearDeterminateWidth();
        }
    }
    disconnectedCallback() {
        clearInterval(this.interval);
    }
    componentWillLoad() {
        this.updateLabel();
        this.el.setAttribute("exportparts", "ic-loading-container");
    }
    componentDidLoad() {
        this.setIndicatorDimensions();
    }
    render() {
        const { theme, label, description, size, fullWidth, innerLabel, monochrome, type, progress, min, max, circularDiameter, indicatorLabel, circularDimensions, } = this;
        const { x, y, r, dashArray, dashOffset } = circularDimensions;
        return (h(Host, { key: '7f78047511c580f116c21d00e76a19dee3319d60', class: {
                [`ic-theme-${theme}`]: theme !== "inherit",
                "ic-loading-indicator-full-width": fullWidth,
                "ic-loading-indicator-label": !!label,
                "ic-loading-indicator-monochrome": monochrome,
            } }, h("div", { key: 'eec896daa35399c26a11873674ba4bc4fd099ddc', class: "ic-loading-container", part: "ic-loading-container" }, h("div", { key: 'b3ab1935e9fe5d50cd6513acd7a8eabb31fadf98', ref: (el) => (this.outerElement = el), class: {
                [`ic-loading-${type}-outer`]: true,
                [progress === undefined ? "indeterminate" : "determinate"]: true,
            }, role: "progressbar", "aria-labelledby": label && size !== "icon" && "ic-loading-label", "aria-label": description, "aria-valuenow": progress, "aria-valuemin": min, "aria-valuemax": max }, h("div", { key: '2f6e1f2e1a88b752cdda2a6b2834441e8546eb9b', ref: (el) => (this.innerElement = el), class: {
                [`ic-loading-${type}-inner`]: true,
                "inner-label": !!innerLabel,
            } }, innerLabel && size === "small" && (h("ic-typography", { key: 'c0d18d282c7090aa2bf478c3beee03e6c2b2f61d', variant: "subtitle-small", class: "inner-text" }, innerLabel)), type === "circular" && (h("svg", { key: 'a5827dba8cff99684667dfa1147186ad0bd3f7a1', class: "ic-loading-circular-svg", viewBox: `0 0 ${circularDiameter} ${circularDiameter}` }, h("circle", { key: 'c53d2155e9aea468427178f5a41e0a24b02565fa', cx: x, cy: y, r: r }), h("circle", { key: 'c5b6aea5b936b056a8103c0dacb81822348b20d1', style: {
                "--circular-steps-max": progress ? `${max}` : undefined,
                "--progress-value": progress !== undefined ? `${progress}` : undefined,
                "--stroke-dasharray": dashArray,
                "--stroke-dashoffset": dashOffset,
            }, cx: x, cy: y, r: r }))))), label && size !== "icon" && (h("ic-typography", { key: 'e586d3c31d518a77bf93f90504318e3a5baeeb64', id: "ic-loading-label", class: "ic-loading-label", role: "status", variant: this.getLabelVariant() }, h("p", { key: 'd1c0b977ef4441ae45f4122a7a0ee03b33b4421b' }, indicatorLabel))))));
    }
    static get is() { return "ic-loading-indicator"; }
    static get encapsulation() { return "shadow"; }
    static get originalStyleUrls() {
        return {
            "$": ["ic-loading-indicator.css"]
        };
    }
    static get styleUrls() {
        return {
            "$": ["ic-loading-indicator.css"]
        };
    }
    static get properties() {
        return {
            "description": {
                "type": "string",
                "mutable": false,
                "complexType": {
                    "original": "string",
                    "resolved": "string",
                    "references": {}
                },
                "required": false,
                "optional": false,
                "docs": {
                    "tags": [],
                    "text": "The description that will be set as the aria-label of the loading indicator when not using a visible label."
                },
                "getter": false,
                "setter": false,
                "attribute": "description",
                "reflect": false,
                "defaultValue": "\"Loading\""
            },
            "fullWidth": {
                "type": "boolean",
                "mutable": false,
                "complexType": {
                    "original": "boolean",
                    "resolved": "boolean",
                    "references": {}
                },
                "required": false,
                "optional": false,
                "docs": {
                    "tags": [],
                    "text": "If `true`, when linear, the full-width variant (i.e. without a border radius) will be displayed."
                },
                "getter": false,
                "setter": false,
                "attribute": "full-width",
                "reflect": true,
                "defaultValue": "false"
            },
            "innerLabel": {
                "type": "number",
                "mutable": false,
                "complexType": {
                    "original": "number",
                    "resolved": "number | undefined",
                    "references": {}
                },
                "required": false,
                "optional": true,
                "docs": {
                    "tags": [{
                            "name": "internal",
                            "text": "The step number of a compact step, managed by ic-step."
                        }],
                    "text": ""
                },
                "getter": false,
                "setter": false,
                "attribute": "inner-label",
                "reflect": false
            },
            "label": {
                "type": "string",
                "mutable": false,
                "complexType": {
                    "original": "string | string[]",
                    "resolved": "string | string[] | undefined",
                    "references": {}
                },
                "required": false,
                "optional": true,
                "docs": {
                    "tags": [],
                    "text": "The label to be displayed beneath the loading indicator.\nDisplay a changing label by supplying an array of messages."
                },
                "getter": false,
                "setter": false,
                "attribute": "label",
                "reflect": false
            },
            "labelDuration": {
                "type": "number",
                "mutable": false,
                "complexType": {
                    "original": "number",
                    "resolved": "number",
                    "references": {}
                },
                "required": false,
                "optional": false,
                "docs": {
                    "tags": [],
                    "text": "The time in milliseconds before the label changes."
                },
                "getter": false,
                "setter": false,
                "attribute": "label-duration",
                "reflect": false,
                "defaultValue": "8000"
            },
            "max": {
                "type": "number",
                "mutable": false,
                "complexType": {
                    "original": "number",
                    "resolved": "number",
                    "references": {}
                },
                "required": false,
                "optional": false,
                "docs": {
                    "tags": [],
                    "text": "The maximum value that the progress value can take.\nUsed to calculate the proportional width of the progress bar."
                },
                "getter": false,
                "setter": false,
                "attribute": "max",
                "reflect": false,
                "defaultValue": "100"
            },
            "min": {
                "type": "number",
                "mutable": false,
                "complexType": {
                    "original": "number",
                    "resolved": "number",
                    "references": {}
                },
                "required": false,
                "optional": false,
                "docs": {
                    "tags": [],
                    "text": "The minimum value that the progress value can take.\nUsed to calculate the proportional width of the progress bar."
                },
                "getter": false,
                "setter": false,
                "attribute": "min",
                "reflect": false,
                "defaultValue": "0"
            },
            "monochrome": {
                "type": "boolean",
                "mutable": false,
                "complexType": {
                    "original": "boolean",
                    "resolved": "boolean",
                    "references": {}
                },
                "required": false,
                "optional": false,
                "docs": {
                    "tags": [],
                    "text": "If `true`, the element will display as black and white."
                },
                "getter": false,
                "setter": false,
                "attribute": "monochrome",
                "reflect": false,
                "defaultValue": "false"
            },
            "progress": {
                "type": "number",
                "mutable": false,
                "complexType": {
                    "original": "number",
                    "resolved": "number | undefined",
                    "references": {}
                },
                "required": false,
                "optional": true,
                "docs": {
                    "tags": [],
                    "text": "The current amount of progress made.\nIf not provided, component acts as an indeterminate loading indicator."
                },
                "getter": false,
                "setter": false,
                "attribute": "progress",
                "reflect": false
            },
            "size": {
                "type": "string",
                "mutable": false,
                "complexType": {
                    "original": "IcLoadingSizes",
                    "resolved": "\"icon\" | \"large\" | \"medium\" | \"small\"",
                    "references": {
                        "IcLoadingSizes": {
                            "location": "import",
                            "path": "./ic-loading-indicator.types",
                            "id": "src/components/ic-loading-indicator/ic-loading-indicator.types.tsx::IcLoadingSizes"
                        }
                    }
                },
                "required": false,
                "optional": false,
                "docs": {
                    "tags": [],
                    "text": "The size of the loading indicator."
                },
                "getter": false,
                "setter": false,
                "attribute": "size",
                "reflect": true,
                "defaultValue": "\"medium\""
            },
            "theme": {
                "type": "string",
                "mutable": false,
                "complexType": {
                    "original": "IcThemeMode",
                    "resolved": "\"dark\" | \"inherit\" | \"light\"",
                    "references": {
                        "IcThemeMode": {
                            "location": "import",
                            "path": "../../utils/types",
                            "id": "src/utils/types.ts::IcThemeMode"
                        }
                    }
                },
                "required": false,
                "optional": false,
                "docs": {
                    "tags": [],
                    "text": "Sets the theme color to the dark or light theme color. \"inherit\" will set the color based on the system settings or ic-theme component."
                },
                "getter": false,
                "setter": false,
                "attribute": "theme",
                "reflect": false,
                "defaultValue": "\"inherit\""
            },
            "type": {
                "type": "string",
                "mutable": false,
                "complexType": {
                    "original": "IcLoadingTypes",
                    "resolved": "\"circular\" | \"linear\"",
                    "references": {
                        "IcLoadingTypes": {
                            "location": "import",
                            "path": "./ic-loading-indicator.types",
                            "id": "src/components/ic-loading-indicator/ic-loading-indicator.types.tsx::IcLoadingTypes"
                        }
                    }
                },
                "required": false,
                "optional": false,
                "docs": {
                    "tags": [],
                    "text": "The type of indicator, either linear or circular."
                },
                "getter": false,
                "setter": false,
                "attribute": "type",
                "reflect": true,
                "defaultValue": "\"circular\""
            }
        };
    }
    static get states() {
        return {
            "circularDiameter": {},
            "circularLineWidth": {},
            "circularDimensions": {},
            "indicatorLabel": {},
            "clipInnerElement": {}
        };
    }
    static get elementRef() { return "el"; }
    static get watchers() {
        return [{
                "propName": "label",
                "methodName": "watchPropHandler"
            }, {
                "propName": "max",
                "methodName": "minMaxChangeHandler"
            }, {
                "propName": "min",
                "methodName": "minMaxChangeHandler"
            }, {
                "propName": "progress",
                "methodName": "watchProgressHandler"
            }, {
                "propName": "type",
                "methodName": "setIndicatorDimensions"
            }];
    }
}
//# sourceMappingURL=ic-loading-indicator.js.map
